@@ -225,6 +225,16 @@ namespace Mono.Debugger
 		}
 	}
 
+	public enum FrameType
+	{
+		Normal,
+		Signal,
+		LMF,
+		Callback,
+		RuntimeInvoke,
+		Special
+	}
+
 	[Serializable]
 	public sealed class StackFrame : DebuggerMarshalByRefObject
 	{
@@ -234,6 +244,7 @@ namespace Mono.Debugger
 		readonly Registers registers;
 
 		int level;
+		FrameType type;
 		Method method;
 		Thread thread;
 		SourceAddress source;
@@ -245,29 +256,31 @@ namespace Mono.Debugger
 		bool has_source;
 		Symbol name;
 
-		internal StackFrame (Thread thread, TargetAddress address, TargetAddress stack_ptr,
-				     TargetAddress frame_address, Registers registers)
+		internal StackFrame (Thread thread, FrameType type, TargetAddress address,
+				     TargetAddress stack_ptr, TargetAddress frame_address,
+				     Registers registers)
 		{
 			this.thread = thread;
+			this.type = type;
 			this.address = address;
 			this.stack_pointer = stack_ptr;
 			this.frame_address = frame_address;
 			this.registers = registers;
 		}
 
-		internal StackFrame (Thread thread, TargetAddress address, TargetAddress stack_ptr,
-				     TargetAddress frame_address, Registers registers,
-				     Language language, Symbol name)
-			: this (thread, address, stack_ptr, frame_address, registers)
+		internal StackFrame (Thread thread, FrameType type, TargetAddress address,
+				     TargetAddress stack_ptr, TargetAddress frame_address,
+				     Registers registers, Language language, Symbol name)
+			: this (thread, type, address, stack_ptr, frame_address, registers)
 		{
 			this.language = language;
 			this.name = name;
 		}
 
-		internal StackFrame (Thread thread, TargetAddress address, TargetAddress stack_ptr,
-				     TargetAddress frame_address, Registers registers,
-				     Method method)
-			: this (thread, address, stack_ptr, frame_address, registers)
+		internal StackFrame (Thread thread, FrameType type, TargetAddress address,
+				     TargetAddress stack_ptr, TargetAddress frame_address,
+				     Registers registers, Method method)
+			: this (thread, type, address, stack_ptr, frame_address, registers)
 		{
 			this.method = method;
 			this.language = method.Module.Language;
@@ -277,10 +290,11 @@ namespace Mono.Debugger
 				this.name = new Symbol (method.Name, address, 0);
 		}
 
-		internal StackFrame (Thread thread, TargetAddress address, TargetAddress stack_ptr,
-				     TargetAddress frame_address, Registers registers,
-				     TargetFunctionType function, SourceLocation location)
-			: this (thread, address, stack_ptr, frame_address, registers)
+		internal StackFrame (Thread thread, FrameType type, TargetAddress address,
+				     TargetAddress stack_ptr, TargetAddress frame_address,
+				     Registers registers, TargetFunctionType function,
+				     SourceLocation location)
+			: this (thread, type, address, stack_ptr, frame_address, registers)
 		{
 			this.function = function;
 			this.language = function.DeclaringType.Language;
@@ -288,10 +302,10 @@ namespace Mono.Debugger
 			this.location = location;
 		}
 
-		internal StackFrame (Thread thread, TargetAddress address, TargetAddress stack_ptr,
-				     TargetAddress frame_address, Registers registers,
-				     Method method, SourceAddress source)
-			: this (thread, address, stack_ptr, frame_address, registers, method)
+		internal StackFrame (Thread thread, FrameType type, TargetAddress address,
+				     TargetAddress stack_ptr, TargetAddress frame_address,
+				     Registers registers, Method method, SourceAddress source)
+			: this (thread, type, address, stack_ptr, frame_address, registers, method)
 		{
 			this.source = source;
 			if (method.HasSource && !method.MethodSource.IsDynamic)
@@ -300,6 +314,10 @@ namespace Mono.Debugger
 			has_source = true;
 		}
 
+		public FrameType Type {
+			get { return type; }
+		}
+
 		public int Level {
 			get { return level; }
 		}
@@ -386,15 +404,13 @@ namespace Mono.Debugger
 			get { return exc_object; }
 		}
 
-		internal void SetExceptionObject (TargetAddress exc)
+		public bool IsManaged {
+			get { return (language != null) && language.IsManaged; }
+		}
+
+		internal void SetExceptionObject (TargetObject exc_object)
 		{
-			try {
-				Language lang = thread.ThreadServant.ProcessServant.MonoLanguage;
-				if (lang != null)
-					exc_object = lang.CreateObject (thread, exc);
-			} catch {
-				exc_object = null;
-			}
+			this.exc_object = exc_object;
 		}
 
 		internal StackFrame UnwindStack (TargetMemoryAccess memory)
