@@ -1,5 +1,6 @@
 using System;
 using System.Text;
+using System.Diagnostics;
 using System.Collections;
 using Mono.Debugger;
 using Mono.Debugger.Languages;
@@ -50,7 +51,6 @@ namespace Mono.Debugger.Frontend
 
 		public void Format (Thread target, TargetObject obj)
 		{
-			sb.Append (String.Format ("({0}) ", obj.TypeName));
 			FormatObjectRecursed (target, obj, false);
 		}
 
@@ -117,6 +117,18 @@ namespace Mono.Debugger.Frontend
 			pos = 0;
 		}
 
+		protected void FormatNullable (Thread target, TargetNullableObject nullable)
+		{
+			bool has_value = nullable.HasValue (target);
+
+			if (has_value) {
+				TargetObject value = nullable.GetValue (target);
+				FormatObjectRecursed (target, value, true);
+			} else {
+				Append ("null");
+			}
+		}
+
 		protected void FormatObjectRecursed (Thread target, TargetObject obj, bool recursed)
 		{
 			try {
@@ -145,6 +157,10 @@ namespace Mono.Debugger.Frontend
 					Format (target, value);
 					break;
 
+				case TargetObjectKind.Nullable:
+					FormatNullable (target, (TargetNullableObject) obj);
+					break;
+
 				default:
 					PrintObject (target, obj);
 					break;
@@ -172,7 +188,7 @@ namespace Mono.Debugger.Frontend
 					TargetObject deref = pobj.GetDereferencedObject (target);
 					Append ("&({0}) ", deref.TypeName);
 					FormatObjectRecursed (target, deref, true);
-				} catch {
+				} catch (Exception ex) {
 					FormatObjectRecursed (target, pobj, true);
 				}
 				break;
@@ -190,7 +206,7 @@ namespace Mono.Debugger.Frontend
 
 			case TargetObjectKind.Class:
 			case TargetObjectKind.Struct:
-				FormatStructObject (target, (TargetStructObject) obj);
+				FormatStructObject (target, (TargetClassObject) obj);
 				break;
 
 			case TargetObjectKind.Fundamental: {
@@ -204,7 +220,11 @@ namespace Mono.Debugger.Frontend
 				break;
 
 			case TargetObjectKind.GenericInstance:
-				FormatStructObject (target, (TargetStructObject) obj);
+				FormatStructObject (target, (TargetClassObject) obj);
+				break;
+
+			case TargetObjectKind.Nullable:
+				FormatNullable (target, (TargetNullableObject) obj);
 				break;
 
 			default:
@@ -213,7 +233,7 @@ namespace Mono.Debugger.Frontend
 			}
 		}
 
-		protected void FormatStructObject (Thread target, TargetStructObject obj)
+		protected void FormatStructObject (Thread target, TargetClassObject obj)
 		{
 			bool first = true;
 
@@ -222,14 +242,14 @@ namespace Mono.Debugger.Frontend
 				FormatClassObject (target, obj, class_info, ref first);
 		}
 
-		protected void FormatClassObject (Thread target, TargetStructObject obj,
+		protected void FormatClassObject (Thread target, TargetClassObject obj,
 						  TargetClass class_info, ref bool first)
 		{
 			Append ("{ ");
 			indent_level += 3;
 
 			if (obj.Type.HasParent) {
-				TargetStructObject parent = obj.GetParentObject (target);
+				TargetClassObject parent = obj.GetParentObject (target);
 				if ((parent != null) && (parent.Type != parent.Type.Language.ObjectType)) {
 					Append ("<{0}> = ", parent.Type.Name);
 					CheckLineWrap ();
@@ -244,6 +264,8 @@ namespace Mono.Debugger.Frontend
 					continue;
 				if (fields [i].IsCompilerGenerated)
 					continue;
+				if (fields [i].DebuggerBrowsableState == DebuggerBrowsableState.Never)
+					continue;
 
 				if (!first) {
 					Append (", ");
