@@ -4,11 +4,12 @@ using NUnit.Framework;
 using Mono.Debugger;
 using Mono.Debugger.Languages;
 using Mono.Debugger.Frontend;
+using Mono.Debugger.Test.Framework;
 
 namespace Mono.Debugger.Tests
 {
-	[TestFixture]
-	public class TestByRef : TestSuite
+	[DebuggerTestFixture]
+	public class TestByRef : DebuggerTestFixture
 	{
 		public TestByRef ()
 			: base ("TestByRef")
@@ -31,23 +32,22 @@ namespace Mono.Debugger.Tests
 
 			int bpt_test = AssertBreakpoint ("Test");
 			AssertExecute ("continue");
-			AssertHitBreakpoint (thread, bpt_test, "X.Test(System.Int32&)", line_test);
+			AssertHitBreakpoint (thread, bpt_test, "X.Test(int&)", line_test);
 
 			AssertExecute ("step");
-			AssertStopped (thread, "X.Test(System.Int32&)", line_test + 1);
+			AssertStopped (thread, "X.Test(int&)", line_test + 1);
 
-			AssertPrint (thread, "foo", "(System.Int32&) &(System.Int32) 3");
+			AssertPrint (thread, "foo", "(int*) &(int) 3");
 			int bpt_unsafe = AssertBreakpoint (line_unsafe);
 
 			AssertExecute ("continue");
 			AssertTargetOutput ("3");
 			AssertNoTargetOutput ();
 
-			AssertHitBreakpoint (thread, bpt_unsafe, "X.UnsafeTest(System.Int32)",
-					     line_unsafe);
+			AssertHitBreakpoint (thread, bpt_unsafe, "X.UnsafeTest(int)", line_unsafe);
 
-			AssertPrint (thread, "ptr", "(System.Int32&) &(System.Int32) 3");
-			AssertPrint (thread, "*ptr", "(System.Int32) 3");
+			AssertPrint (thread, "ptr", "(int*) &(int) 3");
+			AssertPrint (thread, "*ptr", "(int) 3");
 
 			AssertExecute ("continue");
 			AssertTargetOutput ("3");
